require 'mpath_graph'
require 'minitest/autorun'
require 'minitest/unit'

# Author::    Israel Buitron  (mailto:ibuitron@computacion.cs.cinvestav.mx)
class TestMPathGraphOddHole < MiniTest::Unit::TestCase

  def setup
    @oh5_e = [[1,2],[1,5],[2,3],[3,4],[4,5]]
    @oh7_e = [[1,2],[1,7],[2,3],[3,4],[4,5],[5,6],[6,7]]
    @g = [[1,2],[1,5],[2,3],[3,4],[4,5]]
    @doble_c5 = [[0,6],[0,9],[1,2],[1,5],[1,6],[2,3],[3,4],[4,5],[6,7],[7,8],[8,9]]
  end

  def test_is_an_odd_hole?
    # Test with 5-odd-hole
    assert_raises(ArgumentError) { MPathGraph::OddHole::is_an_odd_hole?(nil, @oh5_e, 2) }
    assert_raises(ArgumentError) { MPathGraph::OddHole::is_an_odd_hole?(nil, @oh5_e, 3) }
    assert_raises(ArgumentError) { MPathGraph::OddHole::is_an_odd_hole?((1..6).to_a, @oh5_e, 5) }
    refute MPathGraph::OddHole::is_an_odd_hole?([1,2,3,4,10], @oh5_e, 5) 
    assert MPathGraph::OddHole::is_an_odd_hole?([1,2,3,4,5], @oh5_e, 5)

    # Test with 7-odd-hole
    assert MPathGraph::OddHole::is_an_odd_hole?((1..7).to_a, @oh7_e, 7)
  end

  def test_order_uedge
    assert [1,2], MPathGraph::OddHole::order_uedge(1,2)
    assert [1,2], MPathGraph::OddHole::order_uedge(2,1)
    assert [1,1], MPathGraph::OddHole::order_uedge(1,1)
  end

  def test_rw_search_first
    computed = MPathGraph::OddHole::rw_search_first(@g)
    assert_equal [1,2,3,4,5], computed.sort
  end

  def test_rw_search
    found = MPathGraph::OddHole::rw_search(@doble_c5, holes_limit: 2)
    assert_equal 2, found.size

    # edges = [[1,2],[1,5],[1,6],[1,19],[2,3],[3,4],[3,18],[4,5],
    #   [4,9],[4,18],[5,6],[5,7],[6,7],[7,8],[7,10],[8,9],[8,13],
    #   [10,11],[10,21],[11,12],[12,13],[13,14],[14,15],[15,16],
    #   [16,17],[17,18],[19,20],[20,21]]

    # options = {worm_size: 5, holes_limit: 2}
    # found = MPathGraph::OddHole::rw_search(edges, options)
    # assert_equal 2, found.size

    # options = {worm_size: 7, holes_limit: 1}
    # found = MPathGraph::OddHole::rw_search(edges, options)
    # assert_equal 1, found.size
    # assert_empty found.first-[7, 10, 21, 20, 19, 1, 6]
    # assert_empty [7, 10, 21, 20, 19, 1, 6]-found.first

    # options = {worm_size: 9, holes_limit: 1}
    # found = MPathGraph::OddHole::rw_search(edges, options)
    # assert_equal 1, found.size
    # assert_empty found.first-[18, 4, 9, 8, 13, 14, 15, 16, 17]
    # assert_empty [18, 4, 9, 8, 13, 14, 15, 16, 17]-found.first
  end

  def test_equivalents
    hole = [1,2,3,4,5]
    exp = [[5,4,3,2,1],[2,3,4,5,1],[1,5,4,3,2],[3,4,5,1,2],[2,1,5,4,3],
      [4,5,1,2,3],[3,2,1,5,4],[5,1,2,3,4],[4,3,2,1,5]]
    result = MPathGraph::OddHole::equivalents(hole)
    assert_same exp.size, result.size
    assert [], exp-result
  end
end
